@@ -30,13 +30,13 @@
  * access roles defined for the base #GDataAccessRule (e.g. %GDATA_ACCESS_ROLE_NONE), #GDataDocumentsEntry has its own, such as
  * %GDATA_DOCUMENTS_ACCESS_ROLE_OWNER and %GDATA_DOCUMENTS_ACCESS_ROLE_READER.
  *
- * Documents can (confusingly) be referenced by three different types of IDs: their entry ID, their resource ID and their document ID. Each is a
- * substring of the previous ones (i.e. the entry ID contains the resource ID, which in turn contains the document ID). The resource ID and document ID
- * should almost always be considered as internal, and thus entry IDs (#GDataEntry:id) should normally be used to uniquely identify documents. For more
- * information, see #GDataDocumentsEntry:resource-id.
+ * Documents can (confusingly) be referenced by three different types of IDs: their entry ID, their resource ID and their document ID (untyped resource
+ * ID). Each is a substring of the previous ones (i.e. the entry ID contains the resource ID, which in turn contains the document ID). The resource ID
+ * and document ID should almost always be considered as internal, and thus entry IDs (#GDataEntry:id) should normally be used to uniquely identify
+ * documents. For more information, see #GDataDocumentsEntry:resource-id.
  *
  * For more details of Google Documents' GData API, see the
- * <ulink type="http" url="http://code.google.com/apis/document/docs/2.0/developers_guide_protocol.html">online documentation</ulink>.
+ * <ulink type="http" url="https://developers.google.com/google-apps/documents-list/">online documentation</ulink>.
  *
  * <example>
  * 	<title>Moving an Entry Between Folders</title>
@@ -120,8 +120,9 @@ static void get_xml (GDataParsable *parsable, GString *xml_string);
 static void gdata_documents_entry_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_documents_entry_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static gboolean parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error);
+static gchar *get_entry_uri (const gchar *id);
 
-static const gchar *_get_document_id (GDataDocumentsEntry *self) G_GNUC_PURE;
+static const gchar *_get_untyped_resource_id (GDataDocumentsEntry *self) G_GNUC_PURE;
 
 struct _GDataDocumentsEntryPrivate {
 	gint64 edited;
@@ -130,6 +131,7 @@ struct _GDataDocumentsEntryPrivate {
 	gboolean writers_can_invite;
 	gboolean is_deleted;
 	GDataAuthor *last_modified_by;
+	goffset quota_used; /* bytes */
 };
 
 enum {
@@ -141,6 +143,7 @@ enum {
 	PROP_WRITERS_CAN_INVITE,
 	PROP_ID,
 	PROP_RESOURCE_ID,
+	PROP_QUOTA_USED,
 };
 
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (GDataDocumentsEntry, gdata_documents_entry, GDATA_TYPE_ENTRY,
@@ -151,6 +154,7 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 	GDataParsableClass *parsable_class = GDATA_PARSABLE_CLASS (klass);
+	GDataEntryClass *entry_class = GDATA_ENTRY_CLASS (klass);
 
 	g_type_class_add_private (klass, sizeof (GDataDocumentsEntryPrivate));
 
@@ -164,8 +168,10 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	parsable_class->get_xml = get_xml;
 	parsable_class->get_namespaces = get_namespaces;
 
+	entry_class->get_entry_uri = get_entry_uri;
+
 	/**
-	 * GDataDocumentsEntry:edited
+	 * GDataDocumentsEntry:edited:
 	 *
 	 * The last time the document was edited. If the document has not been edited yet, the content indicates the time it was created.
 	 *
@@ -181,7 +187,7 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	                                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * GDataDocumentsEntry:last-viewed
+	 * GDataDocumentsEntry:last-viewed:
 	 *
 	 * The last time the document was viewed.
 	 *
@@ -221,15 +227,19 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * GDataDocumentsEntry:resource-id
+	 * GDataDocumentsEntry:resource-id:
 	 *
 	 * The resource ID of the document. This should not normally need to be used in client code, and is mostly for internal use. To uniquely
 	 * identify a given document or folder, use its #GDataEntry:id.
 	 *
 	 * Resource IDs have the form:
-	 * <literal><replaceable>document|spreadsheet|presentation|folder</replaceable>:<replaceable>document ID</replaceable></literal>; whereas entry
-	 * IDs have the form:
-	 * <literal>https://docs.google.com/feeds/documents/private/full/<replaceable>resource ID</replaceable></literal>.
+	 * <literal><replaceable>document|spreadsheet|presentation|folder</replaceable>:<replaceable>untyped resource ID</replaceable></literal>; whereas
+	 * entry IDs have the form:
+	 * <literal>https://docs.google.com/feeds/id/<replaceable>resource ID</replaceable></literal> in version 3 of the API.
+	 *
+	 * For more information, see the
+	 * <ulink type="http" url="https://developers.google.com/google-apps/documents-list/#resource_ids_explained">Google Documents
+	 * API reference</ulink>.
 	 *
 	 * Since: 0.11.0
 	 */
@@ -240,9 +250,11 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * GDataDocumentsEntry:document-id
+	 * GDataDocumentsEntry:document-id:
 	 *
-	 * The document ID of the document, which is different from its entry ID (GDataEntry:id).
+	 * The document ID of the document, which is different from its entry ID (GDataEntry:id). The
+	 * <ulink type="http" url="https://developers.google.com/google-apps/documents-list/#terminology_used_in_this_guide">online GData
+	 * Documentation</ulink> refers to these as “untyped resource IDs”.
 	 *
 	 * Since: 0.4.0
 	 * Deprecated: This a substring of the #GDataDocumentsEntry:resource-id, which is more general and should be used instead. (Since: 0.11.0.)
@@ -254,7 +266,7 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
-	 * GDataDocumentsEntry:last-modified-by
+	 * GDataDocumentsEntry:last-modified-by:
 	 *
 	 * Indicates the author of the last modification.
 	 *
@@ -266,10 +278,26 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	                                                      GDATA_TYPE_AUTHOR,
 	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+	/**
+	 * GDataDocumentsEntry:quota-used:
+	 *
+	 * The amount of user quota the document is occupying. Currently, only arbitrary files consume file space quota (whereas standard document
+	 * formats, such as #GDataDocumentsText, #GDataDocumentsSpreadsheet and #GDataDocumentsFolder don't). Measured in bytes.
+	 *
+	 * This property will be <code class="literal">0</code> for documents which aren't consuming any quota.
+	 *
+	 * Since: 0.13.0
+	 */
+	g_object_class_install_property (gobject_class, PROP_QUOTA_USED,
+	                                 g_param_spec_int64 ("quota-used",
+	                                                     "Quota used", "The amount of user quota the document is occupying.",
+	                                                     0, G_MAXINT64, 0,
+	                                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
 	/* Override the ID property since the server returns two different forms of ID depending on how you form a query on an entry. These two forms
-	 * of ID are:
-	 *  - Document ID: /feeds/documents/private/full/[resource_id]
-	 *  - Folder ID: /feeds/folders/private/full/[folder_id]/[resource_id]
+	 * of ID are (for version 3 of the API):
+	 *  - Document ID: /feeds/id/[resource_id]
+	 *  - Folder ID: /feeds/default/private/full/[folder_id]/[resource_id]
 	 * The former is the ID we want; the latter should only ever be used for manipulating the location of documents (i.e. adding them to and
 	 * removing them from folders). The latter will, however, work fine for operations such as updating documents. It's only when one comes to
 	 * try and delete a document that it becomes a problem: sending a DELETE request to the folder ID will only remove the document from that
@@ -362,7 +390,7 @@ gdata_documents_entry_get_property (GObject *object, guint property_id, GValue *
 			g_value_set_string (value, priv->resource_id);
 			break;
 		case PROP_DOCUMENT_ID:
-			g_value_set_string (value, _get_document_id (GDATA_DOCUMENTS_ENTRY (object)));
+			g_value_set_string (value, _get_untyped_resource_id (GDATA_DOCUMENTS_ENTRY (object)));
 			break;
 		case PROP_WRITERS_CAN_INVITE:
 			g_value_set_boolean (value, priv->writers_can_invite);
@@ -379,6 +407,9 @@ gdata_documents_entry_get_property (GObject *object, guint property_id, GValue *
 		case PROP_LAST_MODIFIED_BY:
 			g_value_set_object (value, priv->last_modified_by);
 			break;
+		case PROP_QUOTA_USED:
+			g_value_set_int64 (value, priv->quota_used);
+			break;
 		case PROP_ID: {
 			gchar *uri;
 
@@ -389,7 +420,7 @@ gdata_documents_entry_get_property (GObject *object, guint property_id, GValue *
 			}
 
 			/* Build the ID */
-			uri = _gdata_service_build_uri ("https://docs.google.com/feeds/documents/private/full/%s", priv->resource_id);
+			uri = _gdata_service_build_uri ("https://docs.google.com/feeds/id/%s", priv->resource_id);
 			g_value_take_string (value, uri);
 
 			break;
@@ -413,6 +444,8 @@ gdata_documents_entry_set_property (GObject *object, guint property_id, const GV
 		case PROP_ID:
 			/* Never set an ID (note that this doesn't stop it being set in GDataEntry due to XML parsing) */
 			break;
+		case PROP_QUOTA_USED:
+			/* Read only. */
 		default:
 			/* We don't have any other property... */
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -427,17 +460,19 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 	GDataDocumentsEntry *self = GDATA_DOCUMENTS_ENTRY (parsable);
 
 	if (gdata_parser_is_namespace (node, "http://www.w3.org/2007/app") == TRUE &&
-	    gdata_parser_int64_from_element (node, "edited", P_REQUIRED | P_NO_DUPES, &(self->priv->edited), &success, error) == TRUE) {
+	    gdata_parser_int64_time_from_element (node, "edited", P_REQUIRED | P_NO_DUPES, &(self->priv->edited), &success, error) == TRUE) {
 		return success;
 	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/g/2005") == TRUE) {
-		if (gdata_parser_int64_from_element (node, "lastViewed", P_REQUIRED | P_NO_DUPES,
-		                                     &(self->priv->last_viewed), &success, error) == TRUE ||
+		if (gdata_parser_int64_time_from_element (node, "lastViewed", P_REQUIRED | P_NO_DUPES,
+		                                          &(self->priv->last_viewed), &success, error) == TRUE ||
 		    gdata_parser_object_from_element_setter (node, "feedLink", P_REQUIRED, GDATA_TYPE_LINK,
 		                                             gdata_entry_add_link, self,  &success, error) == TRUE ||
 		    gdata_parser_object_from_element (node, "lastModifiedBy", P_REQUIRED, GDATA_TYPE_AUTHOR,
 		                                      &(self->priv->last_modified_by), &success, error) == TRUE ||
 		    gdata_parser_string_from_element (node, "resourceId", P_REQUIRED | P_NON_EMPTY | P_NO_DUPES, &(self->priv->resource_id),
-		                                      &success, error) == TRUE) {
+		                                      &success, error) == TRUE ||
+		    gdata_parser_int64_from_element (node, "quotaBytesUsed", P_REQUIRED | P_NO_DUPES,
+		                                     &(self->priv->quota_used), 0, &success, error) == TRUE) {
 			return success;
 		} else if (xmlStrcmp (node->name, (xmlChar*) "deleted") ==  0) {
 			/* <gd:deleted> */
@@ -486,6 +521,23 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 	g_hash_table_insert (namespaces, (gchar*) "docs", (gchar*) "http://schemas.google.com/docs/2007");
 }
 
+static gchar *
+get_entry_uri (const gchar *id)
+{
+	const gchar *resource_id;
+
+	/* Version 3: We get an ID similar to “https://docs.google.com/feeds/id/[resource_id]” and want an entry URI
+	 * similar to “https://docs.google.com/feeds/default/private/full/[resource_id]”. */
+	resource_id = g_strrstr (id, "/");
+
+	if (resource_id == NULL) {
+		/* Bail! */
+		return g_strdup (id);
+	}
+
+	return g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/default/private/full", resource_id, NULL);
+}
+
 /**
  * gdata_documents_entry_get_edited:
  * @self: a #GDataDocumentsEntry
@@ -582,23 +634,24 @@ gdata_documents_entry_get_path (GDataDocumentsEntry *self)
 	}
 
 	/* Append the document ID */
-	g_string_append (path, _get_document_id (self));
+	g_string_append (path, _get_untyped_resource_id (self));
 
 	return g_string_free (path, FALSE);
 }
 
-/* Static version so that we can use it internally without triggering deprecation warnings. */
+/* Static version so that we can use it internally without triggering deprecation warnings.
+ * Note that this is what libgdata used to call a "document ID". */
 static const gchar *
-_get_document_id (GDataDocumentsEntry *self)
+_get_untyped_resource_id (GDataDocumentsEntry *self)
 {
 	const gchar *colon;
 
-	/* Document ID should be NULL iff resource ID is. */
+	/* Untyped resource ID should be NULL iff resource ID is. */
 	if (self->priv->resource_id == NULL) {
 		return NULL;
 	}
 
-	/* Resource ID is of the form "document:[document_id]" (or "spreadsheet:[document_id]", etc.),
+	/* Resource ID is of the form "document:[untyped_resource_id]" (or "spreadsheet:[untyped_resource_id]", etc.),
 	 * so we want to return the portion after the colon. */
 	colon = g_utf8_strchr (self->priv->resource_id, -1, ':');
 	g_assert (colon != NULL);
@@ -610,7 +663,9 @@ _get_document_id (GDataDocumentsEntry *self)
  * gdata_documents_entry_get_document_id:
  * @self: a #GDataDocumentsEntry
  *
- * Gets the #GDataDocumentsEntry:document-id property.
+ * Gets the #GDataDocumentsEntry:document-id property. The
+ * <ulink type="http" url="https://developers.google.com/google-apps/documents-list/#terminology_used_in_this_guide">online GData Documentation</ulink>
+ * refers to these as “untyped resource IDs”.
  *
  * Return value: the document's document ID
  *
@@ -622,7 +677,7 @@ gdata_documents_entry_get_document_id (GDataDocumentsEntry *self )
 {
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (self), NULL);
 
-	return _get_document_id (self);
+	return _get_untyped_resource_id (self);
 }
 
 /**
@@ -693,6 +748,24 @@ gdata_documents_entry_get_last_modified_by (GDataDocumentsEntry *self)
 	return self->priv->last_modified_by;
 }
 
+/**
+ * gdata_documents_entry_get_quota_used:
+ * @self: a #GDataDocumentsEntry
+ *
+ * Gets the #GDataDocumentsEntry:quota-used property.
+ *
+ * Return value: the number of quota bytes used by the document
+ *
+ * Since: 0.13.0
+ */
+goffset
+gdata_documents_entry_get_quota_used (GDataDocumentsEntry *self)
+{
+	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (self), 0);
+
+	return self->priv->quota_used;
+}
+
 /**
  * gdata_documents_entry_is_deleted:
  * @self: a #GDataDocumentsEntry
