@@ -27,7 +27,7 @@
  * #GDataDocumentsService is a subclass of #GDataService for communicating with the GData API of Google Documents. It supports querying
  * for, inserting, editing and deleting documents, as well as a folder hierarchy.
  *
- * For more details of Google Documents' GData API, see the <ulink type="http" url="http://code.google.com/apis/document/docs/2.0/reference.html">
+ * For more details of Google Documents' GData API, see the <ulink type="http" url="https://developers.google.com/google-apps/documents-list/">
  * online documentation</ulink>.
  *
  * Fore more details about the spreadsheet downloads handling, see the
@@ -136,7 +136,7 @@
  * </example>
  *
  * The Documents service can be manipulated using batch operations, too. See the
- * <ulink type="http" url="http://code.google.com/apis/documents/docs/2.0/developers_guide_protocol.html#ACLBatch">online documentation on batch
+ * <ulink type="http" url="https://developers.google.com/google-apps/documents-list/#batching_acl_requests">online documentation on batch
  * operations</ulink> for more information.
  *
  * <example>
@@ -269,10 +269,16 @@ gdata_documents_service_error_quark (void)
 	return g_quark_from_static_string ("gdata-documents-service-error-quark");
 }
 
+static void append_query_headers (GDataService *self, GDataAuthorizationDomain *domain, SoupMessage *message);
 static GList *get_authorization_domains (void);
 
+static gchar *_build_v2_upload_uri (GDataDocumentsFolder *folder) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
+static gchar *_get_upload_uri_for_query_and_folder (GDataDocumentsUploadQuery *query,
+                                                    GDataDocumentsFolder *folder) G_GNUC_WARN_UNUSED_RESULT G_GNUC_MALLOC;
+
 _GDATA_DEFINE_AUTHORIZATION_DOMAIN (documents, "writely", "https://docs.google.com/feeds/")
 _GDATA_DEFINE_AUTHORIZATION_DOMAIN (spreadsheets, "wise", "https://spreadsheets.google.com/feeds/")
+_GDATA_DEFINE_AUTHORIZATION_DOMAIN (docs_downloads, "writely", "https://docs.googleusercontent.com/")
 G_DEFINE_TYPE_WITH_CODE (GDataDocumentsService, gdata_documents_service, GDATA_TYPE_SERVICE, G_IMPLEMENT_INTERFACE (GDATA_TYPE_BATCHABLE, NULL))
 
 static void
@@ -280,7 +286,11 @@ gdata_documents_service_class_init (GDataDocumentsServiceClass *klass)
 {
 	GDataServiceClass *service_class = GDATA_SERVICE_CLASS (klass);
 	service_class->feed_type = GDATA_TYPE_DOCUMENTS_FEED;
+
+	service_class->append_query_headers = append_query_headers;
 	service_class->get_authorization_domains = get_authorization_domains;
+
+	service_class->api_version = "3";
 }
 
 static void
@@ -289,6 +299,49 @@ gdata_documents_service_init (GDataDocumentsService *self)
 	/* Nothing to see here */
 }
 
+static void
+append_query_headers (GDataService *self, GDataAuthorizationDomain *domain, SoupMessage *message)
+{
+	g_assert (message != NULL);
+
+	if (message->method == SOUP_METHOD_POST && soup_message_headers_get_one (message->request_headers, "X-Upload-Content-Length") == NULL) {
+		gchar *upload_uri;
+		const gchar *v3_pos;
+
+		upload_uri = soup_uri_to_string (soup_message_get_uri (message), FALSE);
+		v3_pos = strstr (upload_uri, "://docs.google.com/feeds/upload/create-session/default/private/full");
+
+		if (v3_pos != NULL) {
+			gchar *v2_upload_uri;
+			SoupURI *_v2_upload_uri;
+
+			/* Content length header for resumable uploads. Only set it if this looks like the initial request of a resumable upload, and
+			 * if no content length has been set previously.
+			 * This allows methods like gdata_service_insert_entry() (which aren't resumable-upload-aware) to continue working for creating
+			 * documents with metadata only, by simulating the initial request of a resumable upload as described here:
+			 * https://developers.google.com/google-apps/documents-list/#creating_a_new_document_or_file_with_metadata_only */
+			soup_message_headers_replace (message->request_headers, "X-Upload-Content-Length", "0");
+
+			/* Also set the encoding to be content length encoding. */
+			soup_message_headers_set_encoding (message->request_headers, SOUP_ENCODING_CONTENT_LENGTH);
+
+			/* HACK: Work around http://code.google.com/a/google.com/p/apps-api-issues/issues/detail?id=3033 by changing the upload URI
+			 * to the v2 API's upload URI. Grrr. */
+			v2_upload_uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/default/private/full",
+			                             v3_pos + strlen ("://docs.google.com/feeds/upload/create-session/default/private/full"), NULL);
+			_v2_upload_uri = soup_uri_new (v2_upload_uri);
+			soup_message_set_uri (message, _v2_upload_uri);
+			soup_uri_free (_v2_upload_uri);
+			g_free (v2_upload_uri);
+		}
+
+		g_free (upload_uri);
+	}
+
+	/* Chain up to the parent class */
+	GDATA_SERVICE_CLASS (gdata_documents_service_parent_class)->append_query_headers (self, domain, message);
+}
+
 static GList *
 get_authorization_domains (void)
 {
@@ -296,6 +349,7 @@ get_authorization_domains (void)
 
 	authorization_domains = g_list_prepend (authorization_domains, get_documents_authorization_domain ());
 	authorization_domains = g_list_prepend (authorization_domains, get_spreadsheets_authorization_domain ());
+	authorization_domains = g_list_prepend (authorization_domains, get_docs_downloads_authorization_domain ());
 
 	return authorization_domains;
 }
@@ -359,6 +413,14 @@ gdata_documents_service_get_spreadsheet_authorization_domain (void)
 	return get_spreadsheets_authorization_domain ();
 }
 
+static gchar *
+_query_documents_build_request_uri (GDataDocumentsQuery *query)
+{
+	/* If we want to query for documents contained in a folder, the URI is different.
+	 * The "/[folder:id]" suffix is added by the GDataQuery later. */
+	return g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/default/private/full", NULL);
+}
+
 /**
  * gdata_documents_service_query_documents:
  * @self: a #GDataDocumentsService
@@ -398,12 +460,7 @@ gdata_documents_service_query_documents (GDataDocumentsService *self, GDataDocum
 		return NULL;
 	}
 
-	/* If we want to query for documents contained in a folder, the URI is different */
-	if (query != NULL && gdata_documents_query_get_folder_id (query) != NULL)
-		request_uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/folders/private/full", NULL);
-	else
-		request_uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/documents/private/full", NULL);
-
+	request_uri = _query_documents_build_request_uri (query);
 	feed = gdata_service_query (GDATA_SERVICE (self), get_documents_authorization_domain (), request_uri, GDATA_QUERY (query),
 	                            GDATA_TYPE_DOCUMENTS_ENTRY, cancellable, progress_callback, progress_user_data, error);
 	g_free (request_uri);
@@ -456,7 +513,7 @@ gdata_documents_service_query_documents_async (GDataDocumentsService *self, GDat
 		return;
 	}
 
-	request_uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/documents/private/full", NULL);
+	request_uri = _query_documents_build_request_uri (query);
 	gdata_service_query_async (GDATA_SERVICE (self), get_documents_authorization_domain (), request_uri, GDATA_QUERY (query),
 	                           GDATA_TYPE_DOCUMENTS_ENTRY, cancellable, progress_callback, progress_user_data,
 	                           destroy_progress_user_data, callback, user_data);
@@ -465,9 +522,9 @@ gdata_documents_service_query_documents_async (GDataDocumentsService *self, GDat
 
 static GDataUploadStream *
 upload_update_document (GDataDocumentsService *self, GDataDocumentsDocument *document, const gchar *slug, const gchar *content_type,
-                        const gchar *method, const gchar *upload_uri, GCancellable *cancellable)
+                        goffset content_length, const gchar *method, const gchar *upload_uri, GCancellable *cancellable)
 {
-	/* Corrects a bug on spreadsheet content types handling
+	/* HACK: Corrects a bug on spreadsheet content types handling
 	 * The content type for ODF spreadsheets is "application/vnd.oasis.opendocument.spreadsheet" for my ODF spreadsheet;
 	 * but Google Documents' spreadsheet service is waiting for "application/x-vnd.oasis.opendocument.spreadsheet"
 	 * and nothing else.
@@ -476,8 +533,35 @@ upload_update_document (GDataDocumentsService *self, GDataDocumentsDocument *doc
 		content_type = "application/x-vnd.oasis.opendocument.spreadsheet";
 
 	/* We need streaming file I/O: GDataUploadStream */
-	return GDATA_UPLOAD_STREAM (gdata_upload_stream_new (GDATA_SERVICE (self), get_documents_authorization_domain (), method, upload_uri,
-	                                                     GDATA_ENTRY (document), slug, content_type, cancellable));
+	if (content_length == -1) {
+		/* Non-resumable upload. */
+		return GDATA_UPLOAD_STREAM (gdata_upload_stream_new (GDATA_SERVICE (self), get_documents_authorization_domain (), method, upload_uri,
+	                                                             GDATA_ENTRY (document), slug, content_type, cancellable));
+	} else {
+		/* Resumable upload. */
+		return GDATA_UPLOAD_STREAM (gdata_upload_stream_new_resumable (GDATA_SERVICE (self), get_documents_authorization_domain (), method,
+		                                                               upload_uri, GDATA_ENTRY (document), slug, content_type, content_length,
+		                                                               cancellable));
+	}
+}
+
+static gboolean
+_upload_checks (GDataDocumentsService *self, GDataDocumentsDocument *document, GError **error)
+{
+	if (gdata_authorizer_is_authorized_for_domain (gdata_service_get_authorizer (GDATA_SERVICE (self)),
+	                                               get_documents_authorization_domain ()) == FALSE) {
+		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
+		                     _("You must be authenticated to upload documents."));
+		return FALSE;
+	}
+
+	if (document != NULL && gdata_entry_is_inserted (GDATA_ENTRY (document)) == TRUE) {
+		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED,
+		                     _("The document has already been uploaded."));
+		return FALSE;
+	}
+
+	return TRUE;
 }
 
 /**
@@ -494,6 +578,10 @@ upload_update_document (GDataDocumentsService *self, GDataDocumentsDocument *doc
  * the document data does not need to be provided at the moment, just the metadata, use gdata_service_insert_entry() instead (e.g. in the case of
  * creating a new, empty file to be edited at a later date).
  *
+ * This performs a non-resumable upload, unlike gdata_documents_service_upload_document(). This means that errors during transmission will cause the
+ * upload to fail, and the entire document will have to be re-uploaded. It is recommended that gdata_documents_service_upload_document_resumable()
+ * be used instead.
+ *
  * If @document is %NULL, only the document data will be uploaded. The new document entry will be named using @slug, and will have default metadata.
  *
  * The stream returned by this function should be written to using the standard #GOutputStream methods, asychronously or synchronously. Once the stream
@@ -525,26 +613,101 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-	if (gdata_authorizer_is_authorized_for_domain (gdata_service_get_authorizer (GDATA_SERVICE (self)),
-	                                               get_documents_authorization_domain ()) == FALSE) {
-		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-		                     _("You must be authenticated to upload documents."));
+	if (_upload_checks (self, document, error) == FALSE) {
 		return NULL;
 	}
 
-	if (document != NULL && gdata_entry_is_inserted (GDATA_ENTRY (document)) == TRUE) {
-		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED,
-		                     _("The document has already been uploaded."));
-		return NULL;
-	}
-
-	upload_uri = gdata_documents_service_get_upload_uri (folder);
-	upload_stream = upload_update_document (self, document, slug, content_type, SOUP_METHOD_POST, upload_uri, cancellable);
+	/* HACK: Since we're using non-resumable upload, we have to use the v2 API upload URI to work around
+	 * http://code.google.com/a/google.com/p/apps-api-issues/issues/detail?id=3033 */
+	upload_uri = _build_v2_upload_uri (folder);
+	upload_stream = upload_update_document (self, document, slug, content_type, -1, SOUP_METHOD_POST, upload_uri, cancellable);
 	g_free (upload_uri);
 
 	return upload_stream;
 }
 
+/**
+ * gdata_documents_service_upload_document_resumable:
+ * @self: an authenticated #GDataDocumentsService
+ * @document: (allow-none): the #GDataDocumentsDocument to insert, or %NULL
+ * @slug: the filename to give to the uploaded document
+ * @content_type: the content type of the uploaded data
+ * @content_length: the size (in bytes) of the file being uploaded
+ * @query: (allow-none): a query specifying parameters for the upload, or %NULL
+ * @cancellable: (allow-none): a #GCancellable for the entire upload stream, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Uploads a document to Google Documents, using the properties from @document and the document data written to the resulting #GDataUploadStream. If
+ * the document data does not need to be provided at the moment, just the metadata, use gdata_service_insert_entry() instead (e.g. in the case of
+ * creating a new, empty file to be edited at a later date).
+ *
+ * Unlike gdata_documents_service_upload_document(), this method performs a
+ * <ulink type="http" url="http://code.google.com/apis/gdata/docs/resumable_upload.html">resumable upload</ulink> which allows for correction of
+ * transmission errors without re-uploading the entire file. Use of this method is preferred over gdata_documents_service_upload_document().
+ *
+ * If @document is %NULL, only the document data will be uploaded. The new document entry will be named using @slug, and will have default metadata.
+ *
+ * If non-%NULL, the @query specifies parameters for the upload, such as a #GDataDocumentsFolder to upload the document into; and whether to treat
+ * the document as an opaque file, or convert it to a standard format. If @query is %NULL, the document will be uploaded into the root folder, and
+ * automatically converted to a standard format. No OCR or automatic language translation will be performed by default.
+ *
+ * If @query is non-%NULL and #GDataDocumentsUploadQuery:convert is %FALSE, @document must be an instance of #GDataDocumentsDocument. Otherwise,
+ * @document must be a subclass of it, such as #GDataDocumentsPresentation.
+ *
+ * The stream returned by this function should be written to using the standard #GOutputStream methods, asychronously or synchronously. Once the stream
+ * is closed (using g_output_stream_close()), gdata_documents_service_finish_upload() should be called on it to parse and return the updated
+ * #GDataDocumentsDocument for the document. This must be done, as @document isn't updated in-place.
+ *
+ * In order to cancel the upload, a #GCancellable passed in to @cancellable must be cancelled using g_cancellable_cancel(). Cancelling the individual
+ * #GOutputStream operations on the #GDataUploadStream will not cancel the entire upload; merely the write or close operation in question. See the
+ * #GDataUploadStream:cancellable for more details.
+ *
+ * Any upload errors will be thrown by the stream methods, and may come from the #GDataServiceError domain.
+ *
+ * Return value: (transfer full): a #GDataUploadStream to write the document data to, or %NULL; unref with g_object_unref()
+ *
+ * Since: 0.13.0
+ */
+GDataUploadStream *
+gdata_documents_service_upload_document_resumable (GDataDocumentsService *self, GDataDocumentsDocument *document, const gchar *slug,
+                                                   const gchar *content_type, goffset content_length, GDataDocumentsUploadQuery *query,
+                                                   GCancellable *cancellable, GError **error)
+{
+	GDataUploadStream *upload_stream;
+	gchar *upload_uri;
+
+	g_return_val_if_fail (GDATA_IS_DOCUMENTS_SERVICE (self), NULL);
+	g_return_val_if_fail (document == NULL || GDATA_IS_DOCUMENTS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (slug != NULL && *slug != '\0', NULL);
+	g_return_val_if_fail (content_type != NULL && *content_type != '\0', NULL);
+	g_return_val_if_fail (query == NULL || GDATA_IS_DOCUMENTS_UPLOAD_QUERY (query), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	if (_upload_checks (self, document, error) == FALSE) {
+		return NULL;
+	}
+
+	upload_uri = _get_upload_uri_for_query_and_folder (query, NULL);
+	upload_stream = upload_update_document (self, document, slug, content_type, content_length, SOUP_METHOD_POST, upload_uri, cancellable);
+	g_free (upload_uri);
+
+	return upload_stream;
+}
+
+static gboolean
+_update_checks (GDataDocumentsService *self, GError **error)
+{
+	if (gdata_authorizer_is_authorized_for_domain (gdata_service_get_authorizer (GDATA_SERVICE (self)),
+	                                               get_documents_authorization_domain ()) == FALSE) {
+		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
+		                     _("You must be authenticated to update documents."));
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 /**
  * gdata_documents_service_update_document:
  * @self: a #GDataDocumentsService
@@ -557,6 +720,10 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
  * Update the document using the properties from @document and the document data written to the resulting #GDataUploadStream. If the document data does
  * not need to be changed, just the metadata, use gdata_service_update_entry() instead.
  *
+ * This performs a non-resumable upload, unlike gdata_documents_service_update_document(). This means that errors during transmission will cause the
+ * upload to fail, and the entire document will have to be re-uploaded. It is recommended that gdata_documents_service_update_document_resumable()
+ * be used instead.
+ *
  * The stream returned by this function should be written to using the standard #GOutputStream methods, asychronously or synchronously. Once the stream
  * is closed (using g_output_stream_close()), gdata_documents_service_finish_upload() should be called on it to parse and return the updated
  * #GDataDocumentsDocument for the document. This must be done, as @document isn't updated in-place.
@@ -586,17 +753,72 @@ gdata_documents_service_update_document (GDataDocumentsService *self, GDataDocum
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-	if (gdata_authorizer_is_authorized_for_domain (gdata_service_get_authorizer (GDATA_SERVICE (self)),
-	                                               get_documents_authorization_domain ()) == FALSE) {
-		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-		                     _("You must be authenticated to update documents."));
+	if (_update_checks (self, error) == FALSE) {
 		return NULL;
 	}
 
 	update_link = gdata_entry_look_up_link (GDATA_ENTRY (document), GDATA_LINK_EDIT_MEDIA);
 	g_assert (update_link != NULL);
 
-	return upload_update_document (self, document, slug, content_type, SOUP_METHOD_PUT, gdata_link_get_uri (update_link), cancellable);
+	return upload_update_document (self, document, slug, content_type, -1, SOUP_METHOD_PUT, gdata_link_get_uri (update_link),
+	                               cancellable);
+}
+
+/**
+ * gdata_documents_service_update_document_resumable:
+ * @self: a #GDataDocumentsService
+ * @document: the #GDataDocumentsDocument to update
+ * @slug: the filename to give to the uploaded document
+ * @content_type: the content type of the uploaded data
+ * @content_length: the size (in bytes) of the file being uploaded
+ * @cancellable: (allow-none): a #GCancellable for the entire upload stream, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Update the document using the properties from @document and the document data written to the resulting #GDataUploadStream. If the document data does
+ * not need to be changed, just the metadata, use gdata_service_update_entry() instead.
+ *
+ * Unlike gdata_documents_service_update_document(), this method performs a
+ * <ulink type="http" url="http://code.google.com/apis/gdata/docs/resumable_upload.html">resumable upload</ulink> which allows for correction of
+ * transmission errors without re-uploading the entire file. Use of this method is preferred over gdata_documents_service_update_document().
+ *
+ * The stream returned by this function should be written to using the standard #GOutputStream methods, asychronously or synchronously. Once the stream
+ * is closed (using g_output_stream_close()), gdata_documents_service_finish_upload() should be called on it to parse and return the updated
+ * #GDataDocumentsDocument for the document. This must be done, as @document isn't updated in-place.
+ *
+ * In order to cancel the update, a #GCancellable passed in to @cancellable must be cancelled using g_cancellable_cancel(). Cancelling the individual
+ * #GOutputStream operations on the #GDataUploadStream will not cancel the entire update; merely the write or close operation in question. See the
+ * #GDataUploadStream:cancellable for more details.
+ *
+ * Any upload errors will be thrown by the stream methods, and may come from the #GDataServiceError domain.
+ *
+ * For more information, see gdata_service_update_entry().
+ *
+ * Return value: (transfer full): a #GDataUploadStream to write the document data to; unref with g_object_unref()
+ *
+ * Since: 0.13.0
+ */
+GDataUploadStream *
+gdata_documents_service_update_document_resumable (GDataDocumentsService *self, GDataDocumentsDocument *document, const gchar *slug,
+                                                   const gchar *content_type, goffset content_length, GCancellable *cancellable, GError **error)
+{
+	GDataLink *update_link;
+
+	g_return_val_if_fail (GDATA_IS_DOCUMENTS_SERVICE (self), NULL);
+	g_return_val_if_fail (GDATA_IS_DOCUMENTS_DOCUMENT (document), NULL);
+	g_return_val_if_fail (slug != NULL && *slug != '\0', NULL);
+	g_return_val_if_fail (content_type != NULL && *content_type != '\0', NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	if (_update_checks (self, error) == FALSE) {
+		return NULL;
+	}
+
+	update_link = gdata_entry_look_up_link (GDATA_ENTRY (document), GDATA_LINK_RESUMABLE_EDIT_MEDIA);
+	g_assert (update_link != NULL);
+
+	return upload_update_document (self, document, slug, content_type, content_length, SOUP_METHOD_PUT, gdata_link_get_uri (update_link),
+	                               cancellable);
 }
 
 /**
@@ -623,43 +845,44 @@ gdata_documents_service_update_document (GDataDocumentsService *self, GDataDocum
 GDataDocumentsDocument *
 gdata_documents_service_finish_upload (GDataDocumentsService *self, GDataUploadStream *upload_stream, GError **error)
 {
-	const gchar *response_body, *content_type;
+	const gchar *response_body, *term_pos;
 	gssize response_length;
-	GDataEntry *entry;
 	GType new_document_type = G_TYPE_INVALID;
 
-	/* Determine the type of the document we've uploaded */
-	entry = gdata_upload_stream_get_entry (upload_stream);
-	content_type = gdata_upload_stream_get_content_type (upload_stream);
-
-	if (entry != NULL) {
-		new_document_type = G_OBJECT_TYPE (entry);
-	} else if (strcmp (content_type, "application/x-vnd.oasis.opendocument.spreadsheet") == 0 ||
-	           strcmp (content_type, "text/tab-separated-values") == 0 ||
-	           strcmp (content_type, "application/x-vnd.oasis.opendocument.spreadsheet") == 0 ||
-	           strcmp (content_type, "application/vnd.ms-excel") == 0) {
-		new_document_type = GDATA_TYPE_DOCUMENTS_SPREADSHEET;
-	} else if (strcmp (content_type, "application/msword") == 0 ||
-	           strcmp (content_type, "application/vnd.oasis.opendocument.text") == 0 ||
-	           strcmp (content_type, "application/rtf") == 0 ||
-	           strcmp (content_type, "text/html") == 0 ||
-	           strcmp (content_type, "application/vnd.sun.xml.writer") == 0 ||
-	           strcmp (content_type, "text/plain") == 0) {
-		new_document_type = GDATA_TYPE_DOCUMENTS_TEXT;
-	} else if (strcmp (content_type, "application/vnd.ms-powerpoint") == 0) {
-		new_document_type = GDATA_TYPE_DOCUMENTS_PRESENTATION;
-	}
-
-	if (g_type_is_a (new_document_type, GDATA_TYPE_DOCUMENTS_DOCUMENT) == FALSE) {
-		g_set_error (error, GDATA_DOCUMENTS_SERVICE_ERROR, GDATA_DOCUMENTS_SERVICE_ERROR_INVALID_CONTENT_TYPE,
-		             _("The content type of the supplied document ('%s') could not be recognized."), content_type);
-		return NULL;
-	}
-
 	/* Get and parse the response from the server */
 	response_body = gdata_upload_stream_get_response (upload_stream, &response_length);
-	if (response_body == NULL || response_length == 0)
+	if (response_body == NULL || response_length == 0) {
+		/* Error will have been set by the upload stream. */
 		return NULL;
+	}
+
+	/* Hackily determine the document format the server chose, and then parse the XML accordingly. The full parse will pick up any errors in
+	 * our choice of format. */
+	#define TERM_MARKER "<category scheme='http://schemas.google.com/g/2005#kind' term='http://schemas.google.com/docs/2007#"
+	term_pos = g_strstr_len (response_body, response_length, TERM_MARKER);
+	if (term_pos == NULL) {
+		goto done;
+	}
+
+	term_pos += strlen (TERM_MARKER);
+
+	if (g_str_has_prefix (term_pos, "file'") == TRUE) {
+		new_document_type = GDATA_TYPE_DOCUMENTS_DOCUMENT;
+	} else if (g_str_has_prefix (term_pos, "spreadsheet'") == TRUE) {
+		new_document_type = GDATA_TYPE_DOCUMENTS_SPREADSHEET;
+	} else if (g_str_has_prefix (term_pos, "presentation'") == TRUE) {
+		new_document_type = GDATA_TYPE_DOCUMENTS_PRESENTATION;
+	} else if (g_str_has_prefix (term_pos, "document'") == TRUE) {
+		new_document_type = GDATA_TYPE_DOCUMENTS_TEXT;
+	}
+
+done:
+	if (g_type_is_a (new_document_type, GDATA_TYPE_DOCUMENTS_DOCUMENT) == FALSE) {
+		g_set_error (error, GDATA_DOCUMENTS_SERVICE_ERROR, GDATA_DOCUMENTS_SERVICE_ERROR_INVALID_CONTENT_TYPE,
+		             _("The content type of the supplied document ('%s') could not be recognized."),
+		             gdata_upload_stream_get_content_type (upload_stream));
+		return NULL;
+	}
 
 	return GDATA_DOCUMENTS_DOCUMENT (gdata_parsable_new_from_xml (new_document_type, response_body, (gint) response_length, error));
 }
@@ -687,8 +910,8 @@ gdata_documents_service_add_entry_to_folder (GDataDocumentsService *self, GDataD
                                              GCancellable *cancellable, GError **error)
 {
 	GDataDocumentsEntry *new_entry;
-	gchar *uri, *upload_data;
-	const gchar *folder_id;
+	gchar *upload_data;
+	const gchar *uri;
 	SoupMessage *message;
 	guint status;
 
@@ -706,11 +929,9 @@ gdata_documents_service_add_entry_to_folder (GDataDocumentsService *self, GDataD
 	}
 
 	/* NOTE: adding a document to a folder doesn't have server-side ETag support (throws "noPostConcurrency" error) */
-	folder_id = gdata_documents_entry_get_resource_id (GDATA_DOCUMENTS_ENTRY (folder));
-	g_assert (folder_id != NULL);
-	uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/folders/private/full/", folder_id, NULL);
+	uri = gdata_entry_get_content_uri (GDATA_ENTRY (folder));
+	g_assert (uri != NULL);
 	message = _gdata_service_build_message (GDATA_SERVICE (self), get_documents_authorization_domain (), SOUP_METHOD_POST, uri, NULL, TRUE);
-	g_free (uri);
 
 	/* Append the data */
 	upload_data = gdata_parsable_get_xml (GDATA_PARSABLE (entry));
@@ -896,7 +1117,8 @@ gdata_documents_service_remove_entry_from_folder (GDataDocumentsService *self, G
 	g_assert (folder_id != NULL);
 	g_assert (entry_id != NULL);
 
-	uri = _gdata_service_build_uri ("http://docs.google.com/feeds/folders/private/full/%s/%s", folder_id, entry_id);
+	uri = _gdata_service_build_uri ("%s://docs.google.com/feeds/default/private/full/%s/contents/%s", _gdata_service_get_scheme (),
+	                                folder_id, entry_id);
 	message = _gdata_service_build_message (GDATA_SERVICE (self), get_documents_authorization_domain (), SOUP_METHOD_DELETE, uri,
 	                                        gdata_entry_get_etag (GDATA_ENTRY (entry)), TRUE);
 	g_free (uri);
@@ -920,7 +1142,7 @@ gdata_documents_service_remove_entry_from_folder (GDataDocumentsService *self, G
 
 	g_object_unref (message);
 
-	/* Google's servers don't return an updated copy of the entry, so we have to query for it again.
+	/* HACK: Google's servers don't return an updated copy of the entry, so we have to query for it again.
 	 * See: http://code.google.com/p/gdata-issues/issues/detail?id=1380 */
 	return GDATA_DOCUMENTS_ENTRY (gdata_service_query_single_entry (GDATA_SERVICE (self), get_documents_authorization_domain (),
 	                                                                gdata_entry_get_id (GDATA_ENTRY (entry)), NULL,
@@ -1037,6 +1259,37 @@ gdata_documents_service_remove_entry_from_folder_finish (GDataDocumentsService *
 	g_assert_not_reached ();
 }
 
+/* HACK: Work around http://code.google.com/a/google.com/p/apps-api-issues/issues/detail?id=3033 by also using the upload URI for the v2 API. Grrr. */
+static gchar *
+_build_v2_upload_uri (GDataDocumentsFolder *folder)
+{
+	g_return_val_if_fail (folder == NULL || GDATA_IS_DOCUMENTS_FOLDER (folder), NULL);
+
+	/* If we have a folder, return the folder's upload URI */
+	if (folder != NULL) {
+		return _gdata_service_build_uri ("%s://docs.google.com/feeds/default/private/full/%s/contents",
+		                                 _gdata_service_get_scheme (), gdata_documents_entry_get_resource_id (GDATA_DOCUMENTS_ENTRY (folder)));
+	}
+
+	/* Otherwise return the default upload URI */
+	return g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/default/private/full", NULL);
+}
+
+/* NOTE: query may be NULL. */
+static gchar *
+_get_upload_uri_for_query_and_folder (GDataDocumentsUploadQuery *query, GDataDocumentsFolder *folder)
+{
+	if (query == NULL) {
+		query = gdata_documents_upload_query_new ();
+	}
+
+	if (folder != NULL) {
+		gdata_documents_upload_query_set_folder (query, folder);
+	}
+
+	return gdata_documents_upload_query_build_uri (query);
+}
+
 /**
  * gdata_documents_service_get_upload_uri:
  * @folder: (allow-none): the #GDataDocumentsFolder into which to upload the document, or %NULL
@@ -1054,13 +1307,5 @@ gdata_documents_service_get_upload_uri (GDataDocumentsFolder *folder)
 {
 	g_return_val_if_fail (folder == NULL || GDATA_IS_DOCUMENTS_FOLDER (folder), NULL);
 
-	/* If we have a folder, return the folder's upload URI */
-	if (folder != NULL) {
-		const gchar *folder_id = gdata_documents_entry_get_resource_id (GDATA_DOCUMENTS_ENTRY (folder));
-		g_assert (folder_id != NULL);
-		return g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/folders/private/full/", folder_id, NULL);
-	}
-
-	/* Otherwise return the default upload URI */
-	return g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/documents/private/full", NULL);
+	return _get_upload_uri_for_query_and_folder (NULL, folder);
 }
